@@ -21,6 +21,11 @@ _OPT_PATHS = (
     'cloudbaseinit.conf.ec2.EC2Options',
     'cloudbaseinit.conf.maas.MAASOptions',
     'cloudbaseinit.conf.openstack.OpenStackOptions',
+    'cloudbaseinit.conf.azure.AzureOptions',
+    'cloudbaseinit.conf.ovf.OvfOptions',
+    'cloudbaseinit.conf.packet.PacketOptions',
+    'cloudbaseinit.conf.vmwareguestinfo.VMwareGuestInfoConfigOptions',
+    'cloudbaseinit.conf.gce.GCEOptions',
 )
 
 
