@@ -14,7 +14,10 @@
 
 import abc
 
+import six
 
+
+@six.add_metaclass(abc.ABCMeta)
 class BaseUserDataPlugin(object):
 
     def __init__(self, mime_type):
